import setuptools, re, os, sys

with open("README.md", "r") as fh:
    long_description = fh.read()

def delDotPrefix(string):
    '''Delete dot prefix to file extension if present'''
    return string[1:] if string.find(".") == 0 else string

def filesInFolder(folder, fileType):  # Returns list of files of specified file type
    fileType = delDotPrefix(fileType)
    file_regex = re.compile(
        r".*\." + fileType, re.IGNORECASE
    )  # Regular Expression; dot star means find everything
    file_list = []
    for _, _, filenames in os.walk(folder):  # folders, subfolders, filenames
        for singlefile in filenames:
            file_search = file_regex.findall(singlefile)
            if file_search:  # if file_search is not empty
                file_list.append(singlefile)
    return file_list

# NOTE: PATHS CALCULATED IN SETUP.PY NEED UNIX STYLE PATH REFERENCES WITH DIFFERENT SLASHES

setuptools.setup(
    name="$package-name$",
    version="0.0.0",
    author="$author$",
    author_email="$email$",
    description="$description$",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="$url$",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    scripts=[f'cmdline\\{file}' for file in filesInFolder('cmdline/',"*")],     # Scans cmdline folder for files to be used as command-line scripts
    install_requires=[],                                                        # Add package dependencies (e.g. 'numpy')
    include_package_data=True,
)


import datetime, sys, os, shutil, re
from pypi_flow import delDotPrefix, filesInFolder, directoryLastValue, File, replaceWords, packageDir

def LicenseCheck(licenseType):
    if licenseType == "":
        #Find default option listed in License Dictionary
        for k,v in LicenseDict.items(): 
            if "MIT" in v:  #DEFAULT SELECTION
                licenseType = k
    try:
        return LicenseDict[licenseType]
    except:
        print("Error: Invalid licence entry. Aborting project.")
        exit()

# UI
print("\n\nWelcome to the pypi_flow project creator!\n\nA project will be created in this directory based on the following information:\n")

projectFolder = os.getcwd() 
templateSource = packageDir("pypi_flow") + "\\packageTemplate"
licenceTemplateFolder = templateSource + '\\LicenseTemplates'

# COLLECT PROJECT INPUT FROM USER
packageName = input("package name: ")
author = input("author: ")
description = input("project description: ")
email = input("contact email: ")
url = input("project website: ")
year = str(datetime.datetime.now().year)

LicenceFiles = filesInFolder(licenceTemplateFolder)
LicenseDict = {"0":"None",}
i = 1
for file in LicenceFiles:
    tempfilename = directoryLastValue(file).split(".")[0]
    LicenseDict[str(i)] = tempfilename
    i+=1

print("\nSupported project license types:")
for k,v in LicenseDict.items():
    if v == "MIT":
        print(f"{k}) {v} (Default)")
    else:
        print(f"{k}) {v}")

project_license = LicenseCheck(input('\nProceed with: '))
print(f"{project_license}\n")

# REPLACEMENT WORDS DICTIONARY
projectDictionary = {
    "$package-name$":packageName,
    "$author$":author,
    "$description$":description,
    "$email$":email,
    "$url$":url,
    "$year$":year,
}

# CREATE DIRS TO COPY INTO
packageRoot = projectFolder + f"\\{packageName}"
packageFolder = packageRoot + f"\\{packageName}"
cmdlineFolder = packageRoot + f"\\cmdline"
os.makedirs(packageRoot)
os.makedirs(packageFolder)
os.makedirs(cmdlineFolder)

# COPY ALL FILES IN FOLDER - CANNOT USE SHUTIL.COPY(folder) SINCE IT COPIES PERMISSIONS FROM SITE-PACKAGES AND CAUSES ISSUES
tocopyfilelist = filesInFolder(templateSource)
for file in tocopyfilelist:
    try:
        if "pycache" in  file:
            pass
        else:
            shutil.copyfile(file,f"{packageRoot}{file[len(templateSource):]}")
    except:
        pass

# COPY LICENSE AND PACKAGE __init__.py individually
shutil.copyfile(templateSource + '\\package_name\\__init__.py', packageFolder + '\\__init__.py')
if project_license != "None":
    shutil.copyfile(f'{licenceTemplateFolder}\\{project_license}.txt',f"{packageRoot}\\LICENSE")


# REPLACE KEY WORDS IN COPIED TEMPLATES
filelist = filesInFolder(packageRoot)
for file in filelist:
    tempfile = File(file)
    content = tempfile.read()
    content = replaceWords(content, projectDictionary)
    tempfile.write(content)


# Required with every package
# Initializes package upon import call from python
#
# ex.
# import package_name.other_python_file

import re, os


def delDotPrefix(string):
    """Delete dot prefix to file extension if present"""
    return string[1:] if string.find(".") == 0 else string


def filesInFolder(folder, fileType="*"):
    """Returns list of files of specified file type"""
    fileType = delDotPrefix(fileType)
    file_regex = re.compile(
        rf".*\.{fileType}", re.IGNORECASE
    )  # Regular Expression; dot star means find everything
    file_list = []
    for dirpath, _, filenames in os.walk(folder):  # for each folder
        for file in filenames:
            file_search = file_regex.findall(file)
            if file_search:  # if file_search is not empty
                file_list.append(dirpath + "\\" + file)
    return file_list


def replaceWords(textOrFile, dictionary):
    """Replace words in content by dictionary keys and values"""
    base_text = textOrFile
    if os.path.exists(textOrFile):
        base_text = File(textOrFile).read()
    for key, val in dictionary.items():
        try:
            base_text = base_text.replace(key, val)
        except:
            pass
    return base_text


def directoryLastValue(directory):
    """Returns the last value in the directory"""
    return directory.rsplit("\\", 1)[-1]


class File:
    def __init__(self, fileName):
        self.fileName = fileName

    def write(self, content):
        """Overwrites to file - deletes what was there before
        If file did not exist it creates a file"""

        f = open(self.fileName, "w")
        f.write(content)
        f.close()

    def append(self, content, newline=True):
        """Appends content to existing file"""

        f = open(self.fileName, "a")
        f.write("\n" + content) if newline else f.write(content)
        f.close()

    def read(self):
        """Returns existing file content"""

        f = open(self.fileName)
        t = f.read()
        f.close
        return t

    def print(self):
        """Prints existing file content"""

        print(self.read())


def importPackage(package):
    import importlib

    return importlib.import_module(package)


def packagePath(package):
    my_module = importPackage(package)
    return my_module.__file__


def packageDir(package):
    return os.path.dirname(packagePath(package))
